@@ -320,7 +320,7 @@ void QQmlComponentPrivate::clear()
 
     compilationUnit.reset();
     loadedType = {};
-    isInlineComponent = false;
+    inlineComponentName.reset();
 }
 
 QObject *QQmlComponentPrivate::doBeginCreate(QQmlComponent *q, QQmlContext *context)
@@ -335,6 +335,35 @@ QObject *QQmlComponentPrivate::doBeginCreate(QQmlComponent *q, QQmlContext *cont
     return q->beginCreate(context);
 }
 
+static void removePendingQPropertyBinding(
+    QV4::Value *object, const QString &propertyName, QQmlObjectCreator *creator)
+{
+    if (!creator)
+        return;
+
+    QV4::QObjectWrapper *wrapper = object->as<QV4::QObjectWrapper>();
+    if (!wrapper)
+        return;
+
+    QObject *o = wrapper->object();
+    if (!o)
+        return;
+
+    if (QQmlData *ddata = QQmlData::get(o)) {
+        const QQmlPropertyData *propData = ddata->propertyCache->property(
+            propertyName, o, ddata->outerContext);
+        if (propData && propData->isBindable())
+            creator->removePendingBinding(o, propData->coreIndex());
+        return;
+    }
+
+    const QMetaObject *meta = o->metaObject();
+    Q_ASSERT(meta);
+    const int index = meta->indexOfProperty(propertyName.toUtf8());
+    if (index != -1 && meta->property(index).isBindable())
+        creator->removePendingBinding(o, index);
+}
+
 bool QQmlComponentPrivate::setInitialProperty(
         QObject *base, const QString &name, const QVariant &value)
 {
@@ -351,13 +380,16 @@ bool QQmlComponentPrivate::setInitialProperty(
             if (scope.engine->hasException)
                 break;
         }
-        segment = scope.engine->newString(properties.last());
+        const QString lastProperty = properties.last();
+        segment = scope.engine->newString(lastProperty);
         object->put(segment, scope.engine->metaTypeToJS(value.metaType(), value.constData()));
         if (scope.engine->hasException) {
             qmlWarning(base, scope.engine->catchExceptionAsQmlError());
             scope.engine->hasException = false;
             return false;
         }
+
+        removePendingQPropertyBinding(object, lastProperty, state.creator());
         return true;
     }
 
@@ -369,7 +401,12 @@ bool QQmlComponentPrivate::setInitialProperty(
         prop = QQmlProperty(base, name, engine);
     QQmlPropertyPrivate *privProp = QQmlPropertyPrivate::get(prop);
     const bool isValid = prop.isValid();
-    if (!isValid || !privProp->writeValueProperty(value, {})) {
+    if (isValid && privProp->writeValueProperty(value, {})) {
+        if (prop.isBindable()) {
+            if (QQmlObjectCreator *creator = state.creator())
+                creator->removePendingBinding(prop.object(), prop.index());
+        }
+    } else {
         QQmlError error{};
         error.setUrl(url);
         if (isValid) {
@@ -428,13 +465,11 @@ QQmlComponent::~QQmlComponent()
     This property holds the status of component loading. The status can be one of the
     following:
 
-    \list
-    \li \c{Component.Null} - no data is available for the component
-    \li \c{Component.Ready} - the component has been loaded, and can be used to create instances.
-    \li \c{Component.Loading} - the component is currently being loaded
-    \li \c{Component.Error} - an error occurred while loading the component.
+    \value Component.Null       no data is available for the component
+    \value Component.Ready      the component has been loaded, and can be used to create instances.
+    \value Component.Loading    the component is currently being loaded
+    \value Component.Error      an error occurred while loading the component.
         Calling \l errorString() will provide a human-readable description of any errors.
-    \endlist
  */
 
 /*!
@@ -1031,7 +1066,18 @@ QObject *QQmlComponentPrivate::beginCreate(QQmlRefPointer<QQmlContextData> conte
     if (!loadedType.isValid()) {
         enginePriv->referenceScarceResources();
         state.initCreator(std::move(context), compilationUnit, creationContext);
-        rv = state.creator()->create(start, nullptr, nullptr, isInlineComponent ? QQmlObjectCreator::InlineComponent : QQmlObjectCreator::NormalObject);
+
+        QQmlObjectCreator::CreationFlags flags;
+        if (const QString *icName = inlineComponentName.get()) {
+            flags = QQmlObjectCreator::InlineComponent;
+            if (start == -1)
+                start = compilationUnit->inlineComponentId(*icName);
+            Q_ASSERT(start > 0);
+        } else {
+            flags = QQmlObjectCreator::NormalObject;
+        }
+
+        rv = state.creator()->create(start, nullptr, nullptr, flags);
         if (!rv)
             state.appendCreatorErrors();
         enginePriv->dereferenceScarceResources();
@@ -1301,14 +1347,10 @@ void QQmlComponent::loadFromModule(QAnyStringView uri, QAnyStringView typeName,
     } else if (type.isInlineComponentType()) {
         auto baseUrl = type.sourceUrl();
         baseUrl.setFragment(QString());
-        // if the outer type has not been resolved yet, we need to load the outer type synchronously
-        // in order to get the correct object id
-        mode = type.inlineComponentObjectId() > 0 ? mode : QQmlComponent::CompilationMode::PreferSynchronous;
         loadUrl(baseUrl, mode);
         if (!isError()) {
-            d->isInlineComponent = true;
-            d->start = type.inlineComponentObjectId();
-            Q_ASSERT(d->start >= 0);
+            d->inlineComponentName = std::make_unique<QString>(type.elementName());
+            Q_ASSERT(!d->inlineComponentName->isEmpty());
         }
     } else if (type.isSingleton() || type.isCompositeSingleton()) {
         reportError(QLatin1String(R"(%1 is a singleton, and cannot be loaded)")
@@ -1426,6 +1468,13 @@ void QQmlComponentPrivate::incubateObject(
     incubatorPriv->compilationUnit = componentPriv->compilationUnit;
     incubatorPriv->enginePriv = enginePriv;
     incubatorPriv->creator.reset(new QQmlObjectCreator(context, componentPriv->compilationUnit, componentPriv->creationContext));
+
+    if (start == -1) {
+        if (const QString *icName = componentPriv->inlineComponentName.get()) {
+            start = compilationUnit->inlineComponentId(*icName);
+            Q_ASSERT(start > 0);
+        }
+    }
     incubatorPriv->subComponentToCreate = componentPriv->start;
 
     enginePriv->incubate(*incubationTask, forContext);
@@ -1565,7 +1614,10 @@ static void QQmlComponent_setQmlParent(QObject *me, QObject *parent)
 */
 
 
-void QQmlComponentPrivate::setInitialProperties(QV4::ExecutionEngine *engine, QV4::QmlContext *qmlContext, const QV4::Value &o, const QV4::Value &v, RequiredProperties *requiredProperties, QObject *createdComponent)
+void QQmlComponentPrivate::setInitialProperties(
+    QV4::ExecutionEngine *engine, QV4::QmlContext *qmlContext, const QV4::Value &o,
+    const QV4::Value &v, RequiredProperties *requiredProperties, QObject *createdComponent,
+    QQmlObjectCreator *creator)
 {
     QV4::Scope scope(engine);
     QV4::ScopedObject object(scope);
@@ -1605,7 +1657,8 @@ void QQmlComponentPrivate::setInitialProperties(QV4::ExecutionEngine *engine, QV
             qmlWarning(createdComponent, error);
             continue;
         }
-        name = engine->newString(properties.last());
+        const QString lastProperty = properties.last();
+        name = engine->newString(lastProperty);
         object->put(name, val);
         if (engine->hasException) {
             qmlWarning(createdComponent, engine->catchExceptionAsQmlError());
@@ -1614,6 +1667,8 @@ void QQmlComponentPrivate::setInitialProperties(QV4::ExecutionEngine *engine, QV
             auto prop = removePropertyFromRequired(createdComponent, name->toQString(),
                                                    requiredProperties, engine->qmlEngine());
         }
+
+        removePendingQPropertyBinding(object, lastProperty, creator);
     }
 
     engine->hasException = false;
@@ -1699,8 +1754,9 @@ void QQmlComponent::createObject(QQmlV4Function *args)
 
     if (!valuemap->isUndefined()) {
         QV4::Scoped<QV4::QmlContext> qmlContext(scope, v4->qmlContext());
-        QQmlComponentPrivate::setInitialProperties(v4, qmlContext, object, valuemap,
-                                                   d->state.requiredProperties(), rv);
+        QQmlComponentPrivate::setInitialProperties(
+            v4, qmlContext, object, valuemap, d->state.requiredProperties(), rv,
+            d->state.creator());
     }
     if (d->state.hasUnsetRequiredProperties()) {
         QList<QQmlError> errors;
@@ -1869,8 +1925,10 @@ void QQmlComponentPrivate::initializeObjectWithInitialProperties(QV4::QmlContext
     QV4::ScopedValue object(scope, QV4::QObjectWrapper::wrap(v4engine, toCreate));
     Q_ASSERT(object->as<QV4::Object>());
 
-    if (!valuemap.isUndefined())
-        setInitialProperties(v4engine, qmlContext, object, valuemap, requiredProperties, toCreate);
+    if (!valuemap.isUndefined()) {
+        setInitialProperties(
+            v4engine, qmlContext, object, valuemap, requiredProperties, toCreate, state.creator());
+    }
 }
 
 QQmlComponentExtension::QQmlComponentExtension(QV4::ExecutionEngine *v4)
@@ -1969,7 +2027,9 @@ void QV4::QmlIncubatorObject::setInitialState(QObject *o, RequiredProperties *re
         QV4::Scope scope(v4);
         QV4::ScopedObject obj(scope, QV4::QObjectWrapper::wrap(v4, o));
         QV4::Scoped<QV4::QmlContext> qmlCtxt(scope, d()->qmlContext);
-        QQmlComponentPrivate::setInitialProperties(v4, qmlCtxt, obj, d()->valuemap, requiredProperties, o);
+        QQmlComponentPrivate::setInitialProperties(
+            v4, qmlCtxt, obj, d()->valuemap, requiredProperties, o,
+            QQmlIncubatorPrivate::get(d()->incubator)->creator.data());
     }
 }
 
