@@ -215,16 +215,23 @@ void QQmlPrivate::qdeclarativeelement_destructor(QObject *o)
 {
     QObjectPrivate *p = QObjectPrivate::get(o);
     if (QQmlData *d = QQmlData::get(p)) {
-        const auto invalidate = [](QQmlContextData *c) {c->invalidate();};
         if (d->ownContext) {
-            d->ownContext->deepClearContextObject(o, invalidate, invalidate);
+            for (QQmlRefPointer<QQmlContextData> lc = d->ownContext->linkedContext(); lc;
+                 lc = lc->linkedContext()) {
+                lc->invalidate();
+                if (lc->contextObject() == o)
+                    lc->setContextObject(nullptr);
+            }
+            d->ownContext->invalidate();
+            if (d->ownContext->contextObject() == o)
+                d->ownContext->setContextObject(nullptr);
             d->ownContext.reset();
             d->context = nullptr;
-            Q_ASSERT(!d->outerContext || d->outerContext->contextObject() != o);
-        } else if (d->outerContext && d->outerContext->contextObject() == o) {
-            d->outerContext->deepClearContextObject(o, invalidate, invalidate);
         }
 
+        if (d->outerContext && d->outerContext->contextObject() == o)
+            d->outerContext->setContextObject(nullptr);
+
         if (d->hasVMEMetaObject || d->hasInterceptorMetaObject) {
             // This is somewhat dangerous because another thread might concurrently
             // try to resolve the dynamic metaobject. In practice this will then
@@ -251,11 +258,8 @@ void QQmlPrivate::qdeclarativeelement_destructor(QObject *o)
 QQmlData::QQmlData(Ownership ownership)
     : ownMemory(ownership == OwnsMemory), indestructible(true), explicitIndestructibleSet(false),
       hasTaintedV4Object(false), isQueuedForDeletion(false), rootObjectInCreation(false),
-      hasInterceptorMetaObject(false), hasVMEMetaObject(false), hasConstWrapper(false),
-      bindingBitsArraySize(InlineBindingArraySize), notifyList(nullptr),
-      bindings(nullptr), signalHandlers(nullptr), nextContextObject(nullptr), prevContextObject(nullptr),
-      lineNumber(0), columnNumber(0), jsEngineId(0),
-      guards(nullptr), extendedData(nullptr)
+      hasInterceptorMetaObject(false), hasVMEMetaObject(false), hasConstWrapper(false), dummy(0),
+      bindingBitsArraySize(InlineBindingArraySize)
 {
     memset(bindingBitsValue, 0, sizeof(bindingBitsValue));
     init();
@@ -400,7 +404,9 @@ void QQmlData::setQueuedForDeletion(QObject *object)
         if (QQmlData *ddata = QQmlData::get(object)) {
             if (ddata->ownContext) {
                 Q_ASSERT(ddata->ownContext.data() == ddata->context);
-                ddata->ownContext->deepClearContextObject(object);
+                ddata->context->emitDestruction();
+                if (ddata->ownContext->contextObject() == object)
+                    ddata->ownContext->setContextObject(nullptr);
                 ddata->ownContext.reset();
                 ddata->context = nullptr;
             }
@@ -574,16 +580,29 @@ void QQmlEnginePrivate::init()
   \inmodule QtQml
   \brief The QQmlEngine class provides an environment for instantiating QML components.
 
-  A QQmlEngine is used to manage \l{components}{QQmlComponent} and objects created from
-  them and execute their bindings and functions. QQmlEngine also inherits from
-  \l{QJSEngine} which allows seamless integration between your QML components and
-  JavaScript code.
+  Each QML component is instantiated in a QQmlContext.
+  QQmlContext's are essential for passing data to QML
+  components.  In QML, contexts are arranged hierarchically and this
+  hierarchy is managed by the QQmlEngine.
 
-  Each QML component is instantiated in a QQmlContext. In QML, contexts are arranged
-  hierarchically and this hierarchy is managed by the QQmlEngine. By default,
-  components are instantiated in the \l {QQmlEngine::rootContext()}{root context}.
+  Prior to creating any QML components, an application must have
+  created a QQmlEngine to gain access to a QML context.  The
+  following example shows how to create a simple Text item.
 
-  \sa QQmlComponent, QQmlContext, {QML Global Object}, QQmlApplicationEngine
+  \code
+  QQmlEngine engine;
+  QQmlComponent component(&engine);
+  component.setData("import QtQuick 2.0\nText { text: \"Hello world!\" }", QUrl());
+  QQuickItem *item = qobject_cast<QQuickItem *>(component.create());
+
+  //add item to view, etc
+  ...
+  \endcode
+
+  In this case, the Text item will be created in the engine's
+  \l {QQmlEngine::rootContext()}{root context}.
+
+  \sa QQmlComponent, QQmlContext, {QML Global Object}
 */
 
 /*!
@@ -1887,14 +1906,13 @@ QJSValue QQmlEnginePrivate::singletonInstance<QJSValue>(const QQmlType &type)
 {
     Q_Q(QQmlEngine);
 
-    QJSValue value = singletonInstances.value(type);
-    if (!value.isUndefined()) {
-        return value;
-    }
-
-    QQmlType::SingletonInstanceInfo *siinfo = type.singletonInstanceInfo();
+    QQmlType::SingletonInstanceInfo::ConstPtr siinfo = type.singletonInstanceInfo();
     Q_ASSERT(siinfo != nullptr);
 
+    QJSValue value = singletonInstances.value(siinfo);
+    if (!value.isUndefined())
+        return value;
+
     if (siinfo->scriptCallback) {
         value = siinfo->scriptCallback(q, q);
         if (value.isQObject()) {
@@ -1903,7 +1921,7 @@ QJSValue QQmlEnginePrivate::singletonInstance<QJSValue>(const QQmlType &type)
             // should behave identically to QML singleton types.
             q->setContextForObject(o, new QQmlContext(q->rootContext(), q));
         }
-        singletonInstances.convertAndInsert(v4engine(), type, &value);
+        singletonInstances.convertAndInsert(v4engine(), siinfo, &value);
 
     } else if (siinfo->qobjectCallback) {
         QObject *o = siinfo->qobjectCallback(q, q);
@@ -1932,7 +1950,7 @@ QJSValue QQmlEnginePrivate::singletonInstance<QJSValue>(const QQmlType &type)
         }
 
         value = q->newQObject(o);
-        singletonInstances.convertAndInsert(v4engine(), type, &value);
+        singletonInstances.convertAndInsert(v4engine(), siinfo, &value);
     } else if (!siinfo->url.isEmpty()) {
         QQmlComponent component(q, siinfo->url, QQmlComponent::PreferSynchronous);
         if (component.isError()) {
@@ -1943,7 +1961,7 @@ QJSValue QQmlEnginePrivate::singletonInstance<QJSValue>(const QQmlType &type)
         }
         QObject *o = component.beginCreate(q->rootContext());
         value = q->newQObject(o);
-        singletonInstances.convertAndInsert(v4engine(), type, &value);
+        singletonInstances.convertAndInsert(v4engine(), siinfo, &value);
         component.completeCreate();
     }
 
