@@ -582,37 +582,14 @@ bool QQmlImportInstance::resolveType(QQmlTypeLoader *typeLoader, const QHashedSt
         bool ret = uri == typeStr;
         if (ret) {
             Q_ASSERT(!type_return->isValid());
-            auto createICType = [&]() {
-                auto typePriv = new QQmlTypePrivate {QQmlType::RegistrationType::InlineComponentType};
-                bool ok = false;
-                typePriv->extraData.id->objectId = QUrl(this->url).fragment().toInt(&ok);
-                Q_ASSERT(ok);
-                typePriv->extraData.id->url = QUrl(this->url);
-                auto icType = QQmlType(typePriv);
-                typePriv->release();
-                return icType;
-            };
-            if (containingType.isValid()) {
-                // we currently cannot reference a Singleton inside itself
-                // in that case, containingType is still invalid
-                if (int icID = containingType.lookupInlineComponentIdByName(typeStr); icID != -1) {
-                    *type_return = containingType.lookupInlineComponentById(icID);
-                } else {
-                    auto icType = createICType();
-                    int placeholderId = containingType.generatePlaceHolderICId();
-                    const_cast<QQmlImportInstance*>(this)->containingType.associateInlineComponent(typeStr, placeholderId, CompositeMetaTypeIds {}, icType);
-                    *type_return = QQmlType(icType);
-                }
-            } else  {
-                *type_return = createICType();
-            }
+            *type_return = QQmlMetaType::inlineComponentTypeForUrl(QUrl(url));
         }
         return ret;
     }
     QQmlDirComponents::ConstIterator it = qmlDirComponents.find(typeStr), end = qmlDirComponents.end();
     if (it != end) {
         QString componentUrl;
-        bool isCompositeSingleton = false;
+        QQmlMetaType::CompositeTypeLookupMode lookupMode = QQmlMetaType::NonSingleton;
         QQmlDirComponents::ConstIterator candidate = end;
         for ( ; it != end && it.key() == typeStr; ++it) {
             const QQmlDirParser::Component &c = *it;
@@ -657,7 +634,7 @@ bool QQmlImportInstance::resolveType(QQmlTypeLoader *typeLoader, const QHashedSt
 
                     // This is our best candidate so far
                     candidate = it;
-                    isCompositeSingleton = c.singleton;
+                    lookupMode = c.singleton ? QQmlMetaType::Singleton : QQmlMetaType::NonSingleton;
                 }
             }
         }
@@ -665,7 +642,7 @@ bool QQmlImportInstance::resolveType(QQmlTypeLoader *typeLoader, const QHashedSt
         if (candidate != end) {
             if (!base) // ensure we have a componentUrl
                 componentUrl = resolveLocalUrl(QString(url + candidate->typeName + dotqml_string), candidate->fileName);
-            QQmlType returnType = QQmlMetaType::typeForUrl(componentUrl, type, isCompositeSingleton,
+            QQmlType returnType = QQmlMetaType::typeForUrl(componentUrl, type, lookupMode,
                                                            nullptr, candidate->version);
             if (version_return)
                 *version_return = candidate->version;
@@ -714,7 +691,10 @@ bool QQmlImportInstance::resolveType(QQmlTypeLoader *typeLoader, const QHashedSt
                 *typeRecursionDetected = recursion;
             if (recursionRestriction == QQmlImport::AllowRecursion || !recursion) {
                 QQmlType returnType = QQmlMetaType::typeForUrl(
-                        qmlUrl, type, registrationType == QQmlType::CompositeSingletonType, errors);
+                        qmlUrl, type, registrationType == QQmlType::CompositeSingletonType
+                                ? QQmlMetaType::Singleton
+                                : QQmlMetaType::NonSingleton,
+                        errors);
                 if (type_return)
                     *type_return = returnType;
                 return returnType.isValid();
@@ -746,7 +726,7 @@ bool QQmlImports::resolveType(
             *type_return = QQmlMetaType::typeForUrl(
                     resolveLocalUrl(nameSpace->imports.at(0)->url,
                                     unqualifiedtype.toString() + QLatin1String(".qml")),
-                    type, false, errors);
+                    type, QQmlMetaType::NonSingleton, errors);
             return type_return->isValid();
         }
         return false;
@@ -765,23 +745,8 @@ bool QQmlImports::resolveType(
         } else {
             if (resolveTypeInNamespace(splitName.at(0), &m_unqualifiedset, nullptr)) {
                 // either simple type + inline component
-                auto const icName = splitName.at(1).toString();
-                auto objectIndex = type_return->lookupInlineComponentIdByName(icName);
-                if (objectIndex != -1) {
-                    *type_return = type_return->lookupInlineComponentById(objectIndex);
-                } else {
-                    auto icTypePriv = new QQmlTypePrivate(QQmlType::RegistrationType::InlineComponentType);
-                    icTypePriv->setContainingType(type_return);
-                    icTypePriv->extraData.id->url = type_return->sourceUrl();
-                    int placeholderId = type_return->generatePlaceHolderICId();
-                    icTypePriv->extraData.id->url.setFragment(QString::number(placeholderId));
-                    auto icType = QQmlType(icTypePriv);
-                    icTypePriv->release();
-                    type_return->associateInlineComponent(icName, placeholderId, CompositeMetaTypeIds {}, icType);
-                    *type_return = icType;
-                }
-                Q_ASSERT(type_return->containingType().isValid());
-                type_return->setPendingResolutionName(icName);
+                *type_return = QQmlMetaType::inlineComponentTypeForUrl(
+                        type_return->sourceUrl(), splitName.at(1).toString());
                 return true;
             } else {
                 // or a failure
@@ -802,22 +767,8 @@ bool QQmlImports::resolveType(
             error.setDescription(QQmlImportDatabase::tr("- %1 is not a namespace").arg(splitName.at(0).toString()));
         } else {
             if (resolveTypeInNamespace(splitName.at(1), s, nullptr)) {
-                auto const icName = splitName.at(2).toString();
-                auto objectIndex = type_return->lookupInlineComponentIdByName(icName);
-                if (objectIndex != -1)
-                    *type_return = type_return->lookupInlineComponentById(objectIndex);
-                else {
-                    auto icTypePriv = new QQmlTypePrivate(QQmlType::RegistrationType::InlineComponentType);
-                    icTypePriv->setContainingType(type_return);
-                    icTypePriv->extraData.id->url = type_return->sourceUrl();
-                    int placeholderId = type_return->generatePlaceHolderICId();
-                    icTypePriv->extraData.id->url.setFragment(QString::number(placeholderId));
-                    auto icType = QQmlType(icTypePriv);
-                    icTypePriv->release();
-                    type_return->associateInlineComponent(icName, placeholderId, CompositeMetaTypeIds {}, icType);
-                    *type_return = icType;
-                }
-                type_return->setPendingResolutionName(icName);
+                *type_return = QQmlMetaType::inlineComponentTypeForUrl(
+                        type_return->sourceUrl(), splitName.at(2).toString());
                 return true;
             } else {
                 error.setDescription(QQmlImportDatabase::tr("- %1 is not a type").arg(splitName.at(1).toString()));
@@ -1036,26 +987,6 @@ QString QQmlImports::resolvedUri(const QString &dir_arg, QQmlImportDatabase *dat
     return stableRelativePath;
 }
 
-/* removes all file selector occurrences in path
-   firstPlus is the position of the initial '+' in the path
-   which we always have as we check for '+' to decide whether
-   we need to do some work at all
-*/
-static QString pathWithoutFileSelectors(QString path, // we want a copy of path
-                                        qsizetype firstPlus)
-{
-    do {
-        Q_ASSERT(path.at(firstPlus) == u'+');
-        const auto eos = path.size();
-        qsizetype terminatingSlashPos = firstPlus + 1;
-        while (terminatingSlashPos != eos && path.at(terminatingSlashPos) != u'/')
-            ++terminatingSlashPos;
-        path.remove(firstPlus, terminatingSlashPos - firstPlus + 1);
-        firstPlus = path.indexOf(u'+', firstPlus);
-    } while (firstPlus != -1);
-    return path;
-}
-
 /*!
   \internal
 
@@ -1102,47 +1033,17 @@ QTypeRevision QQmlImports::matchingQmldirVersion(
     typedef QQmlDirComponents::const_iterator ConstIterator;
     const QQmlDirComponents &components = qmldir.components();
 
-    QMultiHash<QString, ConstIterator> baseFileName2ConflictingComponents;
-
     ConstIterator cend = components.constEnd();
     for (ConstIterator cit = components.constBegin(); cit != cend; ++cit) {
         for (ConstIterator cit2 = components.constBegin(); cit2 != cit; ++cit2) {
             if (cit2->typeName == cit->typeName && cit2->version == cit->version) {
-                // ugly heuristic to deal with file selectors
-                const auto comp2PotentialFileSelectorPos = cit2->fileName.indexOf(u'+');
-                const bool comp2MightHaveFileSelector = comp2PotentialFileSelectorPos != -1;
-                /* If we detect conflicting paths, we check if they agree when we remove anything looking like a
-                   file selector.
-                   We need to create copies of the filenames, otherwise QString::replace would modify the
-                   existing file-names
-                 */
-                QString compFileName1 = cit->fileName;
-                QString compFileName2 = cit2->fileName;
-                if (auto fileSelectorPos1 = compFileName1.indexOf(u'+'); fileSelectorPos1 != -1) {
-                    // existing entry was file selector entry, fix it up
-                    // it could also be the case that _both_ are using file selectors
-                    QString baseName =  comp2MightHaveFileSelector ? pathWithoutFileSelectors(compFileName2,
-                                                                                         comp2PotentialFileSelectorPos)
-                                                                   : compFileName2;
-                    if (pathWithoutFileSelectors(compFileName1, fileSelectorPos1) == baseName) {
-                        baseFileName2ConflictingComponents.insert(baseName, cit);
-                        baseFileName2ConflictingComponents.insert(baseName, cit2);
-                        continue;
-                    }
-                    // fall through to error case
-                } else if (comp2MightHaveFileSelector) {
-                    // new entry contains file selector (and we now that cit did not)
-                    if (pathWithoutFileSelectors(compFileName2, comp2PotentialFileSelectorPos) == compFileName1) {
-                        baseFileName2ConflictingComponents.insert(compFileName1, cit2);
-                        continue;
-                    }
-                    // fall through to error case
-                }
                 // This entry clashes with a predecessor
                 QQmlError error;
-                error.setDescription(QQmlImportDatabase::tr("\"%1\" version %2.%3 is defined more than once in module \"%4\"")
-                                     .arg(cit->typeName).arg(cit->version.majorVersion())
-                                     .arg(cit->version.minorVersion()).arg(uri));
+                error.setDescription(
+                        QQmlImportDatabase::tr(
+                                "\"%1\" version %2.%3 is defined more than once in module \"%4\"")
+                                .arg(cit->typeName).arg(cit->version.majorVersion())
+                                .arg(cit->version.minorVersion()).arg(uri));
                 errors->prepend(error);
                 return QTypeRevision();
             }
@@ -1151,14 +1052,6 @@ QTypeRevision QQmlImports::matchingQmldirVersion(
         addVersion(cit->version);
     }
 
-    // ensure that all components point to the actual base URL, and let the file selectors resolve them correctly during URL resolution
-    for (auto keyIt = baseFileName2ConflictingComponents.keyBegin(); keyIt != baseFileName2ConflictingComponents.keyEnd(); ++keyIt) {
-        const QString& baseFileName = *keyIt;
-        const auto conflictingComponents = baseFileName2ConflictingComponents.values(baseFileName);
-        for (ConstIterator component: conflictingComponents)
-            component->fileName = baseFileName;
-    }
-
     typedef QList<QQmlDirParser::Script>::const_iterator SConstIterator;
     const QQmlDirScripts &scripts = qmldir.scripts();
 
@@ -1414,6 +1307,15 @@ QTypeRevision QQmlImports::addFileImport(
 
     // The url for the path containing files for this import
     QString url = resolveLocalUrl(m_base, uri);
+    if (url.isEmpty()) {
+        QQmlError error;
+        error.setDescription(
+                QQmlImportDatabase::tr("Cannot resolve URL for import \"%1\"").arg(uri));
+        error.setUrl(m_baseUrl);
+        errors->prepend(error);
+        return QTypeRevision();
+    }
+
     if (!url.endsWith(Slash) && !url.endsWith(Backslash))
         url += Slash;
 
@@ -1531,13 +1433,12 @@ QTypeRevision QQmlImports::updateQmldirContent(
 /*!
  \internal
  */
-bool QQmlImports::addInlineComponentImport(QQmlImportInstance *const importInstance, const QString &name, const QUrl importUrl, QQmlType containingType)
+bool QQmlImports::addInlineComponentImport(QQmlImportInstance *const importInstance, const QString &name, const QUrl importUrl)
 {
     importInstance->url = importUrl.toString();
     importInstance->uri = name;
     importInstance->isInlineComponent = true;
     importInstance->version = QTypeRevision::zero();
-    importInstance->containingType = containingType;
     m_unqualifiedset.imports.push_back(importInstance);
     m_unqualifiedset.setNeedsSorting(true);
     return true;
