@@ -8,11 +8,13 @@
 #include <QtQml/private/qqmljsengine_p.h>
 
 #include <QtCore/qdir.h>
+#include <QtCore/qstring.h>
 
 QT_BEGIN_NAMESPACE
 
 using namespace QQmlJS;
 using namespace QQmlJS::AST;
+using namespace Qt::StringLiterals;
 
 QString toString(const UiQualifiedId *qualifiedId, QChar delimiter = QLatin1Char('.'))
 {
@@ -29,8 +31,7 @@ QString toString(const UiQualifiedId *qualifiedId, QChar delimiter = QLatin1Char
 }
 
 bool QQmlJSTypeDescriptionReader::operator()(
-        QHash<QString, QQmlJSExportedScope> *objects,
-        QStringList *dependencies)
+        QList<QQmlJSExportedScope> *objects, QStringList *dependencies)
 {
     Engine engine;
 
@@ -165,6 +166,7 @@ void QQmlJSTypeDescriptionReader::readDependencies(UiScriptBinding *ast)
 
 void QQmlJSTypeDescriptionReader::readComponent(UiObjectDefinition *ast)
 {
+    m_currentCtorIndex = 0;
     QQmlJSScope::Ptr scope = QQmlJSScope::create();
     QList<QQmlJSScope::Export> exports;
 
@@ -210,7 +212,9 @@ void QQmlJSTypeDescriptionReader::readComponent(UiObjectDefinition *ast)
             } else if (name == QLatin1String("isSingleton")) {
                 scope->setIsSingleton(readBoolBinding(script));
             } else if (name == QLatin1String("isCreatable")) {
-                scope->setIsCreatable(readBoolBinding(script));
+                scope->setCreatableFlag(readBoolBinding(script));
+            } else if (name == QLatin1String("isStructured")) {
+                scope->setStructuredFlag(readBoolBinding(script));
             } else if (name == QLatin1String("isComposite")) {
                 scope->setIsComposite(readBoolBinding(script));
             } else if (name == QLatin1String("hasCustomParser")) {
@@ -241,8 +245,9 @@ void QQmlJSTypeDescriptionReader::readComponent(UiObjectDefinition *ast)
                 addWarning(script->firstSourceLocation(),
                            tr("Expected only name, prototype, defaultProperty, attachedType, "
                               "valueType, exports, interfaces, isSingleton, isCreatable, "
-                              "isComposite, hasCustomParser, exportMetaObjectRevisions, "
-                              "deferredNames, and immediateNames in script bindings, not \"%1\".")
+                              "isStructured, isComposite, hasCustomParser, "
+                              "exportMetaObjectRevisions, deferredNames, and immediateNames "
+                              "in script bindings, not \"%1\".")
                            .arg(name));
             }
         } else {
@@ -258,18 +263,18 @@ void QQmlJSTypeDescriptionReader::readComponent(UiObjectDefinition *ast)
 
     if (metaObjectRevisions)
         checkMetaObjectRevisions(metaObjectRevisions, &exports);
-    m_objects->insert(scope->internalName(), {scope, exports});
+    m_objects->append({scope, exports});
 }
 
-void QQmlJSTypeDescriptionReader::readSignalOrMethod(UiObjectDefinition *ast, bool isMethod,
-                                               const QQmlJSScope::Ptr &scope)
+void QQmlJSTypeDescriptionReader::readSignalOrMethod(
+        UiObjectDefinition *ast, bool isMethod, const QQmlJSScope::Ptr &scope)
 {
     QQmlJSMetaMethod metaMethod;
     // ### confusion between Method and Slot. Method should be removed.
     if (isMethod)
-        metaMethod.setMethodType(QQmlJSMetaMethod::Slot);
+        metaMethod.setMethodType(QQmlJSMetaMethodType::Slot);
     else
-        metaMethod.setMethodType(QQmlJSMetaMethod::Signal);
+        metaMethod.setMethodType(QQmlJSMetaMethodType::Signal);
 
     for (UiObjectMemberList *it = ast->initializer->members; it; it = it->next) {
         UiObjectMember *member = it->member;
@@ -291,8 +296,17 @@ void QQmlJSTypeDescriptionReader::readSignalOrMethod(UiObjectDefinition *ast, bo
                 metaMethod.setReturnTypeName(readStringBinding(script));
             } else if (name == QLatin1String("revision")) {
                 metaMethod.setRevision(readIntBinding(script));
+            } else if (name == QLatin1String("isCloned")) {
+                metaMethod.setIsCloned(true);
             } else if (name == QLatin1String("isConstructor")) {
                 metaMethod.setIsConstructor(true);
+
+                // The constructors in the moc json output are ordered the same
+                // way as the ones in the metaobject. qmltyperegistrar moves them into
+                // the same list as the other members, but maintains their order.
+                metaMethod.setConstructorIndex(
+                            QQmlJSMetaMethod::RelativeFunctionIndex(m_currentCtorIndex++));
+
             } else if (name == QLatin1String("isJavaScriptFunction")) {
                 metaMethod.setIsJavaScriptFunction(true);
             } else if (name == QLatin1String("isList")) {
@@ -304,7 +318,7 @@ void QQmlJSTypeDescriptionReader::readSignalOrMethod(UiObjectDefinition *ast, bo
             } else {
                 addWarning(script->firstSourceLocation(),
                            tr("Expected only name, type, revision, isPointer, isList, "
-                              "isConstructor, and "
+                              "isCloned, isConstructor, and "
                               "isJavaScriptFunction in script bindings."));
             }
         } else {
@@ -351,6 +365,8 @@ void QQmlJSTypeDescriptionReader::readProperty(UiObjectDefinition *ast, const QQ
             property.setIsList(readBoolBinding(script));
         } else if (id == QLatin1String("isFinal")) {
             property.setIsFinal(readBoolBinding(script));
+        } else if (id == QLatin1String("isConstant")) {
+            property.setIsConstant(readBoolBinding(script));
         } else if (id == QLatin1String("revision")) {
             property.setRevision(readIntBinding(script));
         } else if (id == QLatin1String("bindable")) {
@@ -359,6 +375,8 @@ void QQmlJSTypeDescriptionReader::readProperty(UiObjectDefinition *ast, const QQ
             property.setRead(readStringBinding(script));
         } else if (id == QLatin1String("write")) {
             property.setWrite(readStringBinding(script));
+        } else if (id == QLatin1String("reset")) {
+            property.setReset(readStringBinding(script));
         } else if (id == QLatin1String("notify")) {
             property.setNotify(readStringBinding(script));
         } else if (id == QLatin1String("index")) {
@@ -368,8 +386,8 @@ void QQmlJSTypeDescriptionReader::readProperty(UiObjectDefinition *ast, const QQ
         } else {
             addWarning(script->firstSourceLocation(),
                        tr("Expected only type, name, revision, isPointer, isReadonly, isRequired, "
-                          "isFinal, bindable, read, write, notify, index and isList script "
-                          "bindings."));
+                          "isFinal, isList, bindable, read, write, reset, notify, index, and "
+                          "privateClass and script bindings."));
         }
     }
 
@@ -405,9 +423,13 @@ void QQmlJSTypeDescriptionReader::readEnum(UiObjectDefinition *ast, const QQmlJS
             metaEnum.setIsFlag(readBoolBinding(script));
         } else if (name == QLatin1String("values")) {
             readEnumValues(script, &metaEnum);
+        } else if (name == QLatin1String("scoped")) {
+            metaEnum.setScoped(readBoolBinding(script));
+        } else if (name == QLatin1String("type")) {
+            metaEnum.setTypeName(readStringBinding(script));
         } else {
             addWarning(script->firstSourceLocation(),
-                       tr("Expected only name and values script bindings."));
+                       tr("Expected only name, alias, isFlag, values, scoped, or type."));
         }
     }
 
@@ -418,6 +440,9 @@ void QQmlJSTypeDescriptionReader::readParameter(UiObjectDefinition *ast, QQmlJSM
 {
     QString name;
     QString type;
+    bool isConstant = false;
+    bool isPointer = false;
+    bool isList = false;
 
     for (UiObjectMemberList *it = ast->initializer->members; it; it = it->next) {
         UiObjectMember *member = it->member;
@@ -433,18 +458,25 @@ void QQmlJSTypeDescriptionReader::readParameter(UiObjectDefinition *ast, QQmlJSM
         } else if (id == QLatin1String("type")) {
             type = readStringBinding(script);
         } else if (id == QLatin1String("isPointer")) {
-            // ### unhandled
+            isPointer = readBoolBinding(script);
+        } else if (id == QLatin1String("isConstant")) {
+            isConstant = readBoolBinding(script);
         } else if (id == QLatin1String("isReadonly")) {
             // ### unhandled
         } else if (id == QLatin1String("isList")) {
-            // ### unhandled
+            isList = readBoolBinding(script);
         } else {
             addWarning(script->firstSourceLocation(),
-                       tr("Expected only name and type script bindings."));
+                       tr("Expected only name, type, isPointer, isConstant, isReadonly, "
+                          "or IsList script bindings."));
         }
     }
 
-    metaMethod->addParameter(name, type);
+    QQmlJSMetaParameter p(name, type);
+    p.setTypeQualifier(isConstant ? QQmlJSMetaParameter::Const : QQmlJSMetaParameter::NonConst);
+    p.setIsPointer(isPointer);
+    p.setIsList(isList);
+    metaMethod->addParameter(std::move(p));
 }
 
 QString QQmlJSTypeDescriptionReader::readStringBinding(UiScriptBinding *ast)
