@@ -99,8 +99,6 @@ CommentInfo::CommentInfo(QStringView rawComment) : rawComment(rawComment)
         }
         commentEnd = commentBegin + commentStartStr.size();
         quint32 rawEnd = quint32(rawComment.size());
-        while (commentEnd < rawEnd && rawComment.at(commentEnd).isSpace())
-            ++commentEnd;
         commentContentEnd = commentContentBegin = commentEnd;
         QChar e1 = ((expectedEnd.isEmpty()) ? QChar::fromLatin1(0) : expectedEnd.at(0));
         while (commentEnd < rawEnd) {
@@ -115,7 +113,8 @@ CommentInfo::CommentInfo(QStringView rawComment) : rawComment(rawComment)
                         commentContentEnd = commentEnd;
                     }
                 } else {
-                    commentEndStr = rawComment.mid(++commentEnd - 1, 1);
+                    // Comment ends with \n, treat as it is not part of the comment but post whitespace
+                    commentEndStr = rawComment.mid(commentEnd - 1, 1);
                     break;
                 }
             } else if (!c.isSpace()) {
@@ -191,7 +190,7 @@ A comment has methods to write it out again (write) and expose it to the Dom
 /*!
 \brief Expose attributes to the Dom
 */
-bool Comment::iterateDirectSubpaths(DomItem &self, DirectVisitor visitor)
+bool Comment::iterateDirectSubpaths(const DomItem &self, DirectVisitor visitor) const
 {
     bool cont = true;
     cont = cont && self.dvValueField(visitor, Fields::rawComment, rawComment());
@@ -236,7 +235,7 @@ Every region has a name, and should be written out using the OutWriter.writeRegi
 startRegion/ EndRegion). Region comments keeps a mapping containing them.
 */
 
-bool CommentedElement::iterateDirectSubpaths(DomItem &self, DirectVisitor visitor)
+bool CommentedElement::iterateDirectSubpaths(const DomItem &self, DirectVisitor visitor) const
 {
     bool cont = true;
     cont = cont && self.dvWrapField(visitor, Fields::preComments, preComments);
@@ -331,6 +330,9 @@ QSet<int> VisitAll::uiKinds()
                            AST::Node::Kind_UiArrayBinding,     AST::Node::Kind_UiImport,
                            AST::Node::Kind_UiObjectBinding,    AST::Node::Kind_UiObjectDefinition,
                            AST::Node::Kind_UiInlineComponent,  AST::Node::Kind_UiObjectInitializer,
+#if QT_VERSION >= QT_VERSION_CHECK(6, 6, 0)
+                           AST::Node::Kind_UiPragmaValueList,
+#endif
                            AST::Node::Kind_UiPragma,           AST::Node::Kind_UiProgram,
                            AST::Node::Kind_UiPublicMember,     AST::Node::Kind_UiQualifiedId,
                            AST::Node::Kind_UiScriptBinding,    AST::Node::Kind_UiSourceElement,
@@ -346,7 +348,7 @@ public:
     AstRangesVisitor() = default;
 
     void addNodeRanges(AST::Node *rootNode);
-    void addItemRanges(DomItem item, FileLocations::Tree itemLocations, Path currentP);
+    void addItemRanges(const DomItem &item, FileLocations::Tree itemLocations, Path currentP);
 
     void throwRecursionDepthError() override { }
 
@@ -376,7 +378,8 @@ void AstRangesVisitor::addNodeRanges(AST::Node *rootNode)
     AST::Node::accept(rootNode, this);
 }
 
-void AstRangesVisitor::addItemRanges(DomItem item, FileLocations::Tree itemLocations, Path currentP)
+void AstRangesVisitor::addItemRanges(
+        const DomItem &item, FileLocations::Tree itemLocations, Path currentP)
 {
     if (!itemLocations) {
         if (item)
@@ -419,11 +422,7 @@ const QSet<int> AstRangesVisitor::kindsToSkip()
                                              AST::Node::Kind_ClassElementList,
                                              AST::Node::Kind_PatternElementList,
                                              AST::Node::Kind_PatternPropertyList,
-#if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
                                              AST::Node::Kind_TypeArgument,
-#else
-                                             AST::Node::Kind_TypeArgumentList,
-#endif
                                      })
                                    .unite(VisitAll::uiKinds());
     return res;
@@ -434,12 +433,12 @@ const QSet<int> AstRangesVisitor::kindsToSkip()
 \brief Stores the comments associated with javascript AST::Node pointers
 */
 
-bool AstComments::iterateDirectSubpaths(DomItem &self, DirectVisitor visitor)
+bool AstComments::iterateDirectSubpaths(const DomItem &self, DirectVisitor visitor) const
 {
     bool cont = self.dvItemField(visitor, Fields::commentedElements, [this, &self]() {
         return self.subMapItem(Map(
                 self.pathFromOwner().field(Fields::commentedElements),
-                [this](DomItem &map, QString key) {
+                [this](const DomItem &map, QString key) {
                     bool ok;
                     // we expose the comments as map just for debugging purposes,
                     // as key we use the address hex value as key (keys must be strings)
@@ -450,7 +449,7 @@ bool AstComments::iterateDirectSubpaths(DomItem &self, DirectVisitor visitor)
                         return map.wrap(PathEls::Key(key), m_commentedElements[n]);
                     return DomItem();
                 },
-                [this](DomItem &) {
+                [this](const DomItem &) {
                     QSet<QString> res;
                     for (AST::Node *n : m_commentedElements.keys()) {
                         QString name;
@@ -471,7 +470,7 @@ void AstComments::collectComments(MutableDomItem &item)
     if (std::shared_ptr<ScriptExpression> scriptPtr = item.ownerAs<ScriptExpression>()) {
         DomItem itemItem = item.item();
         return collectComments(scriptPtr->engine(), scriptPtr->ast(), scriptPtr->astComments(),
-                               item, FileLocations::treePtr(itemItem));
+                               item, FileLocations::treeOf(itemItem));
     } else if (std::shared_ptr<QmlFile> qmlFilePtr = item.ownerAs<QmlFile>()) {
         return collectComments(qmlFilePtr->engine(), qmlFilePtr->ast(), qmlFilePtr->astComments(),
                                item, qmlFilePtr->fileLocationsTree());
@@ -487,9 +486,9 @@ void AstComments::collectComments(MutableDomItem &item)
 Collects and associates comments with javascript AST::Node pointers and MutableDomItem in
 rootItem
 */
-void AstComments::collectComments(std::shared_ptr<Engine> engine, AST::Node *n,
-                                  std::shared_ptr<AstComments> ccomm, MutableDomItem rootItem,
-                                  FileLocations::Tree rootItemLocations)
+void AstComments::collectComments(
+        std::shared_ptr<Engine> engine, AST::Node *n, std::shared_ptr<AstComments> ccomm,
+        const MutableDomItem &rootItem, FileLocations::Tree rootItemLocations)
 {
     if (!n)
         return;
@@ -504,6 +503,7 @@ void AstComments::collectComments(std::shared_ptr<Engine> engine, AST::Node *n,
         // do not add newline before, but add the one after
         quint32 iPre = cLoc.begin();
         int preNewline = 0;
+        int postNewline = 0;
         QStringView commentStartStr;
         while (iPre > 0) {
             QChar c = code.at(iPre - 1);
@@ -532,8 +532,10 @@ void AstComments::collectComments(std::shared_ptr<Engine> engine, AST::Node *n,
             }
             --iPre;
         }
+
         if (iPre == 0)
             preNewline = 1;
+
         qsizetype iPost = cLoc.end();
         while (iPost < code.size()) {
             QChar c = code.at(iPost);
@@ -546,16 +548,25 @@ void AstComments::collectComments(std::shared_ptr<Engine> engine, AST::Node *n,
                 } else {
                     break;
                 }
+            } else {
+                if (c == QLatin1Char('\n')) {
+                    ++postNewline;
+                    if (iPost + 1 < code.size() && code.at(iPost + 1) == QLatin1Char('\n')) {
+                        ++iPost;
+                        ++postNewline;
+                    }
+                } else if (c == QLatin1Char('\r')) {
+                    if (iPost + 1 < code.size() && code.at(iPost + 1) == QLatin1Char('\n')) {
+                        ++iPost;
+                        ++postNewline;
+                    }
+                }
             }
             ++iPost;
-            if (c == QLatin1Char('\n'))
+            if (postNewline > 1)
                 break;
-            if (c == QLatin1Char('\r')) {
-                if (iPost < code.size() && code.at(iPost) == QLatin1Char('\n'))
-                    ++iPost;
-                break;
-            }
         }
+
         ElementRef commentEl;
         bool pre = true;
         auto iStart = ranges.starts.lowerBound(cLoc.begin());
@@ -654,6 +665,7 @@ void AstComments::collectComments(std::shared_ptr<Engine> engine, AST::Node *n,
                 commentEl.size = n->lastSourceLocation().end() - n->firstSourceLocation().begin();
             }
         }
+
         Comment comment(code.mid(iPre, iPost - iPre), preNewline);
         if (commentEl.element.index() == 0 && std::get<0>(commentEl.element)) {
             CommentedElement &cEl = commentedElements[std::get<0>(commentEl.element)];
@@ -662,7 +674,7 @@ void AstComments::collectComments(std::shared_ptr<Engine> engine, AST::Node *n,
             else
                 cEl.postComments.append(comment);
         } else if (commentEl.element.index() == 1) {
-            DomItem rComments = rootItem.item()
+            MutableDomItem rComments = rootItem.item()
                                         .path(std::get<1>(commentEl.element).path)
                                         .field(Fields::comments);
             if (RegionComments *rCommentsPtr = rComments.mutableAs<RegionComments>()) {
@@ -717,7 +729,7 @@ QMultiMap<quint32, const QList<Comment> *> AstComments::allCommentsInNode(AST::N
     return v.nodeComments;
 }
 
-bool RegionComments::iterateDirectSubpaths(DomItem &self, DirectVisitor visitor)
+bool RegionComments::iterateDirectSubpaths(const DomItem &self, DirectVisitor visitor) const
 {
     bool cont = true;
     if (!regionComments.isEmpty())
