@@ -3,10 +3,7 @@
 
 #include "qquickgraphicsconfiguration_p.h"
 #include <QCoreApplication>
-
-#if QT_CONFIG(vulkan)
-#include <QtGui/private/qrhivulkan_p.h>
-#endif
+#include <rhi/qrhi.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -57,8 +54,10 @@ QT_BEGIN_NAMESPACE
     Vulkan, or graphics APIs where the concept is applicable. Where some
     concepts are not applicable, the related settings are simply ignored.
 
-    Examples of functions in this category are preferredInstanceExtensions()
-    and setDeviceExtensions().
+    Examples of functions in this category are setDeviceExtensions() and
+    preferredInstanceExtensions(). The latter is useful when the application
+    manages its own \l QVulkanInstance which is then associated with the
+    QQuickWindow via \l QWindow::setVulkanInstance().
 
     \section1 Qt Quick Scene Graph Renderer Configuration
 
@@ -181,7 +180,8 @@ QT_BEGIN_NAMESPACE
     - With pipeline cache saving enabled, Qt stores all render and compute
     pipelines encountered into an MTLBinaryArchive. Saving the pipeline cache
     stores the blob retrieved from the archive, with additional metadata to
-    identify the device.
+    identify the device. \b{Note:} currently MTLBinaryArchive usage is disabled
+    on macOS and iOS due to various issues on some hardware and OS versions.
 
     \li OpenGL - There is no native concept of pipelines, the "pipeline cache"
     stores a collection of program binaries retrieved via
@@ -572,6 +572,51 @@ bool QQuickGraphicsConfiguration::isDebugMarkersEnabled() const
     return d->flags.testFlag(QQuickGraphicsConfigurationPrivate::EnableDebugMarkers);
 }
 
+/*!
+    When enabled, GPU timing data is collected from command buffers on
+    platforms and 3D APIs where this is supported. This data is then printed in
+    the renderer logs that can be enabled via \c{QSG_RENDER_TIMING} environment
+    variable or logging categories such as \c{qt.scenegraph.time.renderloop},
+    and may also be made visible to other modules, such as Qt Quick 3D's
+    \l DebugView item.
+
+    By default this is disabled, because collecting the data may involve
+    additional work, such as inserting timestamp queries in the command stream,
+    depending on the underlying graphics API. To enable, either call this
+    function with \a enable set to true, or set the \c{QSG_RHI_PROFILE}
+    environment variable to a non-zero value.
+
+    Graphics APIs where this can be expected to be supported are Direct 3D 11,
+    Direct 3D 12, Vulkan (as long as the underlying Vulkan implementation
+    supports timestamp queries), Metal, and OpenGL with a core or compatibility
+    profile context for version 3.3 or newer. Timestamps are not supported with
+    OpenGL ES.
+
+    \since 6.6
+
+    \sa timestampsEnabled(), setDebugMarkers()
+ */
+void QQuickGraphicsConfiguration::setTimestamps(bool enable)
+{
+    if (d->flags.testFlag(QQuickGraphicsConfigurationPrivate::EnableTimestamps) != enable) {
+        detach();
+        d->flags.setFlag(QQuickGraphicsConfigurationPrivate::EnableTimestamps, enable);
+    }
+}
+
+/*!
+    \return true if GPU timing collection is enabled.
+
+    By default the value is false.
+
+    \since 6.6
+    \sa setTimestamps()
+ */
+bool QQuickGraphicsConfiguration::timestampsEnabled() const
+{
+    return d->flags.testFlag(QQuickGraphicsConfigurationPrivate::EnableTimestamps);
+}
+
 /*!
     Requests choosing an adapter or physical device that uses software-based
     rasterization. Applicable only when the underlying API has support for
@@ -780,9 +825,9 @@ QQuickGraphicsConfigurationPrivate::QQuickGraphicsConfigurationPrivate()
     if (enableDebugLayer)
         flags |= EnableDebugLayer;
 
-    static const bool enableDebugMarkers = qEnvironmentVariableIntValue("QSG_RHI_PROFILE");
-    if (enableDebugMarkers)
-        flags |= EnableDebugMarkers;
+    static const bool enableProfilingRelated = qEnvironmentVariableIntValue("QSG_RHI_PROFILE");
+    if (enableProfilingRelated)
+        flags |= EnableDebugMarkers | EnableTimestamps;
 
     static const bool preferSoftwareDevice = qEnvironmentVariableIntValue("QSG_RHI_PREFER_SOFTWARE_RENDERER");
     if (preferSoftwareDevice)
