@@ -225,6 +225,8 @@ void QQuickRangeSliderNode::setHandle(QQuickItem *handle)
     if (d->handle == handle)
         return;
 
+    QQuickControlPrivate::warnIfCustomizationNotSupported(d->slider, handle, QStringLiteral("handle"));
+
     if (!d->handle.isExecuting())
         d->cancelHandle();
 
@@ -354,6 +356,7 @@ public:
 
     void itemImplicitWidthChanged(QQuickItem *item) override;
     void itemImplicitHeightChanged(QQuickItem *item) override;
+    void itemDestroyed(QQuickItem *item) override;
 
     bool live = true;
     qreal from = defaultFrom;
@@ -526,22 +529,21 @@ bool QQuickRangeSliderPrivate::handleRelease(const QPointF &point, ulong timesta
         return true;
     QQuickRangeSliderNodePrivate *pressedNodePrivate = QQuickRangeSliderNodePrivate::get(pressedNode);
 
-    if (q->keepMouseGrab() || q->keepTouchGrab()) {
-        const qreal oldPos = pressedNode->position();
-        qreal pos = positionAt(q, pressedNode->handle(), point);
-        if (snapMode != QQuickRangeSlider::NoSnap)
-            pos = snapPosition(q, pos);
-        qreal val = valueAt(q, pos);
-        if (!qFuzzyCompare(val, pressedNode->value()))
-            pressedNode->setValue(val);
-        else if (snapMode != QQuickRangeSlider::NoSnap)
-            pressedNodePrivate->setPosition(pos);
-        q->setKeepMouseGrab(false);
-        q->setKeepTouchGrab(false);
+    const qreal oldPos = pressedNode->position();
+    qreal pos = positionAt(q, pressedNode->handle(), point);
+    if (snapMode != QQuickRangeSlider::NoSnap)
+        pos = snapPosition(q, pos);
+    qreal val = valueAt(q, pos);
+    if (!qFuzzyCompare(val, pressedNode->value()))
+        pressedNode->setValue(val);
+    else if (snapMode != QQuickRangeSlider::NoSnap)
+        pressedNodePrivate->setPosition(pos);
+    q->setKeepMouseGrab(false);
+    q->setKeepTouchGrab(false);
+
+    if (!qFuzzyCompare(pressedNode->position(), oldPos))
+        emit pressedNode->moved();
 
-        if (!qFuzzyCompare(pressedNode->position(), oldPos))
-            emit pressedNode->moved();
-    }
     pressedNode->setPressed(false);
     pressedNodePrivate->touchId = -1;
     return true;
@@ -595,6 +597,15 @@ void QQuickRangeSliderPrivate::itemImplicitHeightChanged(QQuickItem *item)
         emit second->implicitHandleHeightChanged();
 }
 
+void QQuickRangeSliderPrivate::itemDestroyed(QQuickItem *item)
+{
+    QQuickControlPrivate::itemDestroyed(item);
+    if (item == first->handle())
+        first->setHandle(nullptr);
+    else if (item == second->handle())
+        second->setHandle(nullptr);
+}
+
 QQuickRangeSlider::QQuickRangeSlider(QQuickItem *parent)
     : QQuickControl(*(new QQuickRangeSliderPrivate), parent)
 {
