@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+
 #include <qtest.h>
 
 #include <QSignalSpy>
@@ -120,7 +121,7 @@ private slots:
 Q_DECLARE_METATYPE(QList<QQmlError>)
 
 tst_QQuickLoader::tst_QQuickLoader()
-    : QQmlDataTest(QT_QMLTEST_DATADIR)
+    : QQmlDataTest(QT_QMLTEST_DATADIR, FailOnWarningsPolicy::FailOnWarnings)
 {
     qmlRegisterType<SlowComponent>("LoaderTest", 1, 0, "SlowComponent");
     qRegisterMetaType<QList<QQmlError>>();
@@ -705,7 +706,7 @@ void tst_QQuickLoader::initialPropertyValues()
 
     ThreadedTestHTTPServer server(dataDirectory());
 
-    foreach (const QString &warning, expectedWarnings)
+    for (const QString &warning : std::as_const(expectedWarnings))
         QTest::ignoreMessage(QtWarningMsg, QRegularExpression(warning.toLatin1().constData()));
 
     QQmlEngine engine;
@@ -764,7 +765,7 @@ void tst_QQuickLoader::initialPropertyValuesError()
     QFETCH(QUrl, qmlFile);
     QFETCH(QStringList, expectedWarnings);
 
-    foreach (const QString &warning, expectedWarnings)
+    for (const QString &warning : std::as_const(expectedWarnings))
         QTest::ignoreMessage(QtWarningMsg, warning.toUtf8().constData());
 
     QQmlEngine engine;
@@ -919,7 +920,7 @@ void tst_QQuickLoader::asynchronous()
     QQuickLoader *loader = root->findChild<QQuickLoader*>("loader");
     QVERIFY(loader);
 
-    foreach (const QString &warning, expectedWarnings)
+    for (const QString &warning : std::as_const(expectedWarnings))
         QTest::ignoreMessage(QtWarningMsg, warning.toUtf8().constData());
 
     QVERIFY(!loader->item());
@@ -1492,7 +1493,7 @@ void tst_QQuickLoader::setSourceAndCheckStatus()
     QMetaObject::invokeMethod(loader, "load", Q_ARG(QVariant, QVariant::fromValue(QStringLiteral(""))));
     QCOMPARE(loader->status(), QQuickLoader::Null);
 
-    QMetaObject::invokeMethod(loader, "load", Q_ARG(QVariant, QVariant()));
+    QMetaObject::invokeMethod(loader, "load", Q_ARG(QVariant, QVariant(QUrl())));
     QCOMPARE(loader->status(), QQuickLoader::Null);
 }
 
