@@ -102,9 +102,6 @@ private slots:
     void focusMultiplePopup();
     void contentChildrenChange();
     void doubleClickInMouseArea();
-    void fadeDimmer_data();
-    void fadeDimmer();
-    void noDimmer();
 
 private:
     static bool hasWindowActivation();
@@ -230,10 +227,6 @@ void tst_QQuickPopup::overlay_data()
 
 void tst_QQuickPopup::overlay()
 {
-#ifdef Q_OS_ANDROID
-    QSKIP("Test crashes. See QTBUG-118532");
-#endif
-
     QFETCH(QString, source);
     QFETCH(bool, modal);
     QFETCH(bool, dim);
@@ -2268,10 +2261,6 @@ void tst_QQuickPopup::contentChildrenChange()
 
 void tst_QQuickPopup::doubleClickInMouseArea()
 {
-#ifdef Q_OS_ANDROID
-    QSKIP("Test crashes. See QTBUG-118532");
-#endif
-
     QQuickView window;
     QVERIFY(QQuickTest::showView(window, testFileUrl("doubleClickInMouseArea.qml")));
 
@@ -2290,64 +2279,6 @@ void tst_QQuickPopup::doubleClickInMouseArea()
     QCOMPARE(longPressSpy.count(), 0);
 }
 
-void tst_QQuickPopup::fadeDimmer_data()
-{
-    QTest::addColumn<bool>("modality");
-
-    QTest::addRow("modal") << true;
-    QTest::addRow("modeless") << true;
-}
-
-void tst_QQuickPopup::fadeDimmer()
-{
-    QFETCH(const bool, modality);
-    QQuickApplicationHelper helper(this, "fadeDimmer.qml");
-    QVERIFY2(helper.ready, helper.failureMessage());
-
-    QQuickWindow *window = helper.window;
-    window->show();
-    QVERIFY(QTest::qWaitForWindowExposed(window));
-
-    auto *popup = window->contentItem()->findChild<QQuickPopup *>();
-    QVERIFY(popup);
-
-    popup->setModal(modality);
-    popup->open();
-    auto dimmer = QQuickPopupPrivate::get(popup)->dimmer;
-    QVERIFY(dimmer);
-    int opacityChangeCount = 0;
-    connect(dimmer, &QQuickItem::opacityChanged, this, [&opacityChangeCount]{
-        ++opacityChangeCount;
-    });
-    QTRY_VERIFY(popup->isOpened());
-    QTRY_COMPARE(dimmer->opacity(), popup->property("dimmerOpacity").toDouble());
-    QCOMPARE_GT(opacityChangeCount, 2);
-
-    opacityChangeCount = 0;
-    popup->setVisible(false);
-    QTRY_VERIFY(!popup->isVisible());
-    QCOMPARE_GT(opacityChangeCount, 2);
-}
-
-void tst_QQuickPopup::noDimmer()
-{
-    QQuickApplicationHelper helper(this, "noDimmer.qml");
-    QVERIFY2(helper.ready, helper.failureMessage());
-
-    QQuickWindow *window = helper.window;
-    window->show();
-    QVERIFY(QTest::qWaitForWindowExposed(window));
-
-    auto *drawer = window->contentItem()->findChild<QQuickDrawer *>();
-    QVERIFY(drawer);
-
-    drawer->open();
-    auto dimmer = QQuickPopupPrivate::get(drawer)->dimmer;
-    QVERIFY(dimmer);
-    // this must not crash
-    QTRY_VERIFY(!drawer->isModal());
-}
-
 QTEST_QUICKCONTROLS_MAIN(tst_QQuickPopup)
 
 #include "tst_qquickpopup.moc"
